@@ -1,10 +1,10 @@
 mod statsf64;
 mod statsi64;
-mod vecf64impls;
+mod vecf64;
 mod indices;
-mod vecu8impls;
-mod mutvecimpls;
-mod vecvecimpls;
+mod vecu8;
+mod mutvec;
+mod vecvec;
 pub mod functions;
 
 use crate::functions::GI; 
@@ -22,7 +22,7 @@ impl std::fmt::Display for Med {
     fn fmt(&self, f: &mut std::fmt::Formatter) -> std::fmt::Result {
         write!(
             f,
-            "(Lower Q: {}, Median: {}, Upper Q: {})",
+            "median:\n\tLower Q: {}\n\tMedian:  {}\n\tUpper Q: {}",
             GI(self.lquartile),
             GI(self.median),
             GI(self.uquartile)
@@ -96,6 +96,8 @@ pub trait Vecf64 {
     fn sadd(self, s: f64) -> Vec<f64>; 
     /// Scalar product of two vectors
     fn dotp(self, v: &[f64]) -> f64;
+    /// Inverse vecor of magnitude 1/|v|
+    fn vinverse(self) -> Vec<f64>;
     /// Cosine = a.dotp(b)/(a.vmag*b.vmag)
     fn cosine(self, _v: &[f64]) -> f64; 
     /// Vector subtraction
@@ -108,7 +110,8 @@ pub trait Vecf64 {
     fn vmagsq(self) -> f64;
     /// Euclidian distance between two points
     fn vdist(self, v: &[f64]) -> f64;
-    
+    /// vdist between two points squared
+    fn vdistsq(self, v: &[f64]) -> f64;   
     /// Unit vector
     fn vunit(self) -> Vec<f64>;
     /// Area of parallelogram between two vectors (magnitude of cross product)
@@ -148,9 +151,9 @@ pub trait Vecu8 {
     fn vmagsq(self) -> f64;
     /// Probability density function of bytes data
     fn pdf(self) -> Vec<f64>;
-    /// Counts of joint bytes values
     /// Information (entropy) in nats of &[u8]
     fn entropy(self) -> f64;
+    /// Counts of joint bytes values
     fn jointpdf(self, v:&[u8]) -> Vec<Vec<u32>>;
     /// Joint entropy of &[u8],&[u8] in nats 
     fn jointentropy(self, v:&[u8]) -> f64;
@@ -163,14 +166,15 @@ pub trait Vecu8 {
     /// Scalar product of u8 and f64 vectors
     fn dotp(self, v: &[f64]) -> f64;
     /// Scalar product of two u8 vectors
-    fn dotpu8(self, v: &[u8]) -> u64;
+     fn dotpu8(self, v: &[u8]) -> u64;
     /// Cosine between two positive u8 vectors
     fn cosineu8(self, v: &[u8]) -> f64;
     /// Area proportional to the swept arc
     fn varc(self, v:&[f64]) -> f64;
     /// Euclidian distance 
-    fn vdist(self, v: &[f64]) -> f64;  
-    
+    fn vdist(self, v: &[f64]) -> f64;
+    /// Euclidian distance between byte vectors
+    fn vdistsq(self, v: &[u8]) -> u64;    
 }
 
 /// Mutable vector operations.
@@ -205,8 +209,10 @@ pub trait VecVecu8 {
 /// Methods applicable to vector of vectors of <f64>
 pub trait VecVec {
 
-    /// Centroid = euclidian mean of a set of points
+    /// Arithmetic Centroid = euclidian mean of a set of points
     fn acentroid(self) -> Vec<f64>;
+    /// Harmonic Centroid = harmonic mean of a set of points
+    fn hcentroid(self) -> Vec<f64>; 
     /// Sums of distances from each point to all other points
     fn distsums(self) -> Vec<f64>;
     /// Sum of distances from one point given by indx
@@ -217,11 +223,13 @@ pub trait VecVec {
     fn dists(self, v: &[f64]) -> Vec<f64>;
     /// Medoid and Outlier (by distance) of a set of points
     fn medoid(self) -> (f64, usize, f64, usize);
-
+ 
     /// Eccentricity vectors from each point
     fn eccentricities(self) -> Vec<Vec<f64>>;
     /// Ecentricity scalar measure of an internal point given by indx
     fn eccentrinset(self, indx: usize) -> f64;
+    /// Eccentricity vector for a non member point
+    fn eccnonmember(self, p:&[f64]) -> Vec<f64>;
     /// Eccentricity scalar measure and vector of any point     
     fn veccentr(self, thisp: &[f64]) -> (f64, Vec<f64>);
     /// Eccentricity scalar measure only, of any point
@@ -239,11 +247,17 @@ pub trait VecVec {
     fn nmedian(self, eps: f64) -> Vec<f64>;
     /// Betterpoint gives new approximation to nmedian
     fn betterpoint(self, v: &[f64]) -> (f64, Vec<f64>);
+    /// First iteration point for geometric medians
+    fn firstpoint(self) -> Vec<f64>;
     /// Trend between two sets
     fn trend(self, eps: f64, v: Vec<Vec<f64>>) -> Vec<f64>;
     /// Subtract m from all points - e.g. transform to zero median form
     fn translate(self, m: &[f64]) -> Vec<Vec<f64>>;
-
+    /// Vector iterative two point method for finding the geometric median
+    fn gmedian(self, eps: f64) -> Vec<f64>;
+    /// Secant nethod for finding the geometric median
+    fn smedian(self, eps: f64) -> Vec<f64>;
+ 
 }
 
 /// Methods to manipulate indices of Vec<usize> type
